;*****************************************************************************
; AddLight - rozjasnia kazda skladowa obrazka o okreslony poziom (0-31)
; Wejscie:
;   ESI - adres obrazka do rozjasnienia
;   EDI - adres bufora do zapisu
;   EBP - poziom natezenia, ktory trzeba dodac
;   EDX - ilosc pixeli do przeksztalcenia
; Wyjscie:
;   brak
;*****************************************************************************
AddLight        Proc
        cmp     ebp, 0
        jle     al_exit

        mov     edi, virtual_screen     ; zrodlo i cel to virtual_screen
        mov     esi, edi                ;
        mov     edx, 320*200            ; 64000 pixeli

al_pet:
        lodsw

        mov     ebx, eax
        and     eax, 0ffe0h
        and     ebx, 1fh
        add     ebx, ebp
        cmp     ebx, 1fh
        jng     skip_r
        mov     ebx, 1fh
skip_r:
        or      eax, ebx

        mov     ebx, eax
        and     eax, 0f81fh
        shl     ebp, 6                  ; skladowa g jest (0-63), a nie (0-31)
        and     ebx, 7e0h
        add     ebx, ebp
        cmp     ebx, 7e0h
        jng     skip_g
        mov     ebx, 7e0h
skip_g:
        or      eax, ebx

        mov     ebx, eax
        and     eax, 07ffh
        shl     ebp, 5
        and     ebx, 0f800h
        add     ebx, ebp
        cmp     ebx, 0f800h
        jng     skip_b
        mov     ebx, 0f800h
skip_b:
        or      eax, ebx

        stosw

        shr     ebp, 11
        dec     edx
        jnz     al_pet

al_exit:
        ret
AddLight        EndP

;*****************************************************************************
; Makro pomocnicze do ponizszej procki
;*****************************************************************************
cien    Macro skladowa
        xor     ecx, ecx
        mov     cl, skladowa
        mov     temp, ecx
        fild    temp
        mov     temp, 63
        fidiv   temp
        mov     temp, edx
        fimul   temp
        frndint
        fistp   temp
EndM

;*****************************************************************************
; Tworzy tablice cieniowania dla obrazka
; Wejscie:
;   ESI - paleta obrazka
;   EDI - adres tablicy cieniowania
; Wyjscie:
;   brak
;*****************************************************************************
skl_r   db 0
skl_g   db 0
skl_b   db 0
calc_shade_tbl  Proc
        finit
        mov     ebp, 256                ; 256 kolorow do cieniowania
cst_pet_zew:
        lodsb
        mov     skl_r, al
        lodsb
        mov     skl_g, al
        lodsb
        mov     skl_b, al

        xor     edx, edx
cst_pet_wew:
        cien    skl_r
        mov     eax, temp
        cien    skl_g
        mov     ebx, temp
        cien    skl_b
        mov     ecx, temp
        call    MakeCol16
        stosw

        inc     edx
        cmp     edx, 64                 ; 64 odcienie kazdego koloru
        jne     cst_pet_wew

        dec     ebp
        jnz     cst_pet_zew

        ret
calc_shade_tbl  EndP

;*****************************************************************************
; Tworzy tablice cieniowania dla obrazka
; Wejscie:
;   ESI - paleta obrazka
;   EDI - adres tablicy cieniowania
; Wyjscie:
;   brak
;*****************************************************************************
cien48  Macro skladowa
        xor     ecx, ecx
        mov     cl, skladowa
        mov     temp, ecx
        fild    temp
        mov     temp, 15
        fidiv   temp
        mov     temp, edx
        fimul   temp
        frndint
        fistp   temp
EndM

cien16  Macro skladowa
        mov     ecx, 63
        sub     cl, skladowa
        mov     temp, ecx
        fild    temp
        mov     temp, 47
        fidiv   temp
        mov     temp, edx
        fimul   temp
        frndint
        fistp   temp
EndM

calc_bump_shade_tbl  Proc
        finit
        mov     ebp, 256                ; 256 kolorow do cieniowania
cstb_pet_zew:
        lodsb
        mov     skl_r, al
        lodsb
        mov     skl_g, al
        lodsb
        mov     skl_b, al

        xor     edx, edx
cstb_pet_wew:
        cien48  skl_r
        mov     eax, temp
        cien48  skl_g
        mov     ebx, temp
        cien48  skl_b
        mov     ecx, temp
        call    MakeCol16
        stosw

        inc     edx
        cmp     edx, 16                 ; 16 odcieni kazdego koloru
        jne     cstb_pet_wew

        xor     edx, edx
cstb_pet_wew2:
        cien16  skl_r
        mov     eax, temp
        add     al, skl_r

        cien16  skl_g
        mov     ebx, temp
        add     bl, skl_g

        cien16  skl_b
        mov     ecx, temp
        add     cl, skl_b

        call    MakeCol16
        stosw

        inc     edx
        cmp     edx, 48                 ; + 48 odcieni do bialego
        jne     cstb_pet_wew2

        dec     ebp
        jnz     cstb_pet_zew

        ret
calc_bump_shade_tbl  EndP

;*****************************************************************************
; Pokazuje obrazek o okreslonym poziomie cieniowania
; Wejscie:
;   EBX - poziom cieniowania
;   ESI - obrazek
;   EBP - shade table
; Wyjscie:
;   brak
;*****************************************************************************
show_shade_pic  Proc
        mov     edi, virtual_screen
        add     edi, 48*320*2 + 96*2
        mov     ecx, 64
ssp_pet_zew:
        mov     eax, 128
ssp_pet_wew:
        xor     edx, edx
        mov     dl, [esi]
        sal     edx, 7                  ; mnozenie przez 128 = 64*2 (bo word)
        add     edx, ebx
        add     edx, ebx
        mov     dx, [ebp + edx]
        mov     [edi], dx

        add     edi, 2
        inc     esi
        dec     eax
        jnz     ssp_pet_wew

        add     edi, (320-128)*2

        dec     ecx
        jnz     ssp_pet_zew

        ret
show_shade_pic  EndP
